#include "renderer.h"
#include "simplescene.h"

namespace Msp {
namespace GL {

void SimpleScene::add(Renderable &r)
{
	// Add to cache as well if the cache is valid
	if(renderables.insert(&r).second && !cache.empty())
		cache.push_back(&r);
}

void SimpleScene::remove(Renderable &r)
{
	renderables.erase(&r);
	cache.clear();
}

void SimpleScene::populate_cache() const
{
	if(cache.empty() && !renderables.empty())
	{
		cache.reserve(renderables.size());
		cache.insert(cache.end(), renderables.begin(), renderables.end());
	}
}

void SimpleScene::setup_frame(Renderer &renderer)
{
	populate_cache();
	for(RenderableArray::const_iterator i=cache.begin(); i!=cache.end(); ++i)
		(*i)->setup_frame(renderer);
}

void SimpleScene::finish_frame()
{
	for(RenderableArray::const_iterator i=cache.begin(); i!=cache.end(); ++i)
		(*i)->finish_frame();
}

void SimpleScene::render(Renderer &renderer, Tag tag) const
{
	populate_cache();
	if(setup_frustum(renderer))
	{
		for(RenderableArray::const_iterator i=cache.begin(); i!=cache.end(); ++i)
			if(!frustum_cull(**i))
				renderer.render(**i, tag);
	}
	else
	{
		for(RenderableArray::const_iterator i=cache.begin(); i!=cache.end(); ++i)
			renderer.render(**i, tag);
	}
}

} // namespace GL
} // namespace Msp
